# -*- coding: utf-8 -*-

# FOGLAMP_BEGIN
# See: https://foglamp-foglamp-documentation.readthedocs-hosted.com
# FOGLAMP_END

import numpy as np

import pywt
from pyvib import signal as vibsignal


def denoise(data, wavelet='db6', level=1, n_freq=None):
    """
    1. Adapted from waveletSmooth function found here:
    http://connor-johnson.com/2016/01/24/using-pywavelets-to-remove-high-frequency-noise/
    2. Threshold equation and using hard mode in threshold as mentioned
    in section '3.2 denoising based on optimized singular values' from paper by Tomas Vantuch:
    http://dspace.vsb.cz/bitstream/handle/10084/133114/VAN431_FEI_P1807_1801V001_2018.pdf
    """
    
    def maddest(d, axis=None):
        """
        Mean Absolute Deviation
        """
        return np.mean(np.absolute(d - np.mean(d, axis)), axis)

    # Decompose to get the wavelet coefficients
    coeff = pywt.wavedec( data, wavelet, mode="per" )
    
    # Calculate sigma for threshold as defined in http://dspace.vsb.cz/bitstream/handle/10084/133114/VAN431_FEI_P1807_1801V001_2018.pdf
    # As noted by @harshit92 MAD referred to in the paper is Mean Absolute Deviation not Median Absolute Deviation
    sigma = maddest( coeff[-level] )  * (1/0.6745)

    # Calculte the univeral threshold
    uthresh = sigma * np.sqrt( 2*np.log( len( data ) ) )
    coeff[1:] = ( pywt.threshold( i, value=uthresh, mode='hard' ) for i in coeff[1:] )

    # Reconstruct the signal using the thresholded coefficients
    return pywt.waverec( coeff, wavelet, mode='per' )



def autocorrelate(data, fs):
    if True:
        return vibsignal.autocorrelation(data, fs, normalize=True)[:data.shape[0]]
    else:
        result = signal.correlate(data, data, mode='full')/data.shape[0]   # valid, same
        return result[len(result)//2:]
